/*
 * io.h
 */

#ifndef IO_H_
#define IO_H_

typedef void (*iocallback_t)(int fd, short events, void *opaque);

enum {
	EVENT_READ = 0x01,
	EVENT_WRITE = 0x02,
	EVENT_ERROR = 0x04,
	EVENT_TIMEOUT = 0x08
};

struct io_context {
	int 		current_desc;	/* fd of the callback we are running now */
	size_t 		nfds_size;
	size_t 		nfds_used;	/* number of pollfd structures in fdset */
	struct pollfd	*fdset;		/* this is what poll is interested in */
	size_t		desc_size;	/* number of elements in desc */
	struct fd_desc	**desc;		/* pointers to descriptors indexed by fd  */
	int 		fd_deleted_flag :1;	/* indicates fd was removed from set */
	int		reduse_mem_flag :1;
	int		loop_running :1;
};

struct io_context *io_new();
void io_free(struct io_context **io);
int io_add(struct io_context *io, int fd, short events, iocallback_t callback, void *opaque);
void io_change(struct io_context *io, int fd, short events);
void io_del(struct io_context *io, int fd);
int io_loop_once(struct io_context *io);
void io_loop_run(struct io_context *io);
void io_loop_stop(struct io_context *io);

#endif
